@@ -31,7 +31,7 @@ import glob
 import re
 import traceback
 
-latest_duplicity = '0.6.08b'
+latest_duplicity = '0.6.10'
 
 temp_dir = None
 gconf_dir = None
@@ -165,7 +165,7 @@ def create_local_config(dest='/'):
   if dest is None:
     dest = get_temp_name('local')
     os.system('mkdir -p %s' % dest)
-  elif dest[0] != '/':
+  elif dest[0] != '/' and dest.find(':') == -1:
     dest = os.getcwd()+'/'+dest
   set_gconf_value("backend", "file")
   set_gconf_value("file/path", dest)
@@ -216,15 +216,15 @@ def get_temp_name(extra):
   create_temp_dir()
   return temp_dir + '/' + extra
 
-def create_mount(path=None, mtype='ext3', size=20):
+def create_mount(path=None, mtype='ext', size=20):
   global cleanup_mounts
-  if mtype is None: mtype = 'ext3'
+  if mtype is None or mtype == 'ext': mtype = 'ext4'
   if size is None: size = 20
   if path is None:
     path = get_temp_name('blob')
     if not os.path.exists(path):
       os.system('dd if=/dev/zero of=%s bs=1 count=0 seek=%dM' % (path, size))
-      if mtype == 'ext3':
+      if mtype.startswith('ext'):
         args = '-F'
       else:
         args = ''
@@ -248,32 +248,49 @@ def run(method):
   success = False
   try:
     success = method()
+    if success is None:
+      success = True # for tests that use exceptions as errors and return nothing
   except:
     traceback.print_exc()
   finally:
     quit()
     cleanup(success)
 
+def dup_meets_version(major, minor, micro):
+  # replicates logic in DuplicityInfo a bit
+  dupver = subprocess.Popen(['duplicity', '--version'], stdout=subprocess.PIPE).communicate()[0].strip().split()[1]
+  if dupver == '999': return True
+  dupmajor, dupminor, dupmicro = dupver.split('.')
+  dupmajor = int(dupmajor)
+  dupminor = int(dupminor)
+  # dupmicro = int(dupmicro) # sometimes micro has weird characters like 'b' in it
+  if dupmajor > major:  return True
+  if dupmajor < major:  return False
+  if dupminor > minor:  return True
+  if dupminor < minor:  return False
+  if dupmicro >= micro: return True
+  else:                 return False
+
 def get_manifest_date(filename):
   return re.sub('.*\.([0-9TZ]+)\.manifest.*', '\\1', filename)
 
-def list_manifests():
-  destdir = get_temp_name('local')
+def list_manifests(dest='local'):
+  destdir = get_temp_name(dest)
   files = sorted(glob.glob('%s/*.manifest*' % destdir), key=get_manifest_date)
   if not files:
     raise Exception("Expected manifest, found none")
   files = filter(lambda x: x.count('duplicity-full') == 0 or x.count('.to.') == 0, files) # don't get the in-between manifests
   return (destdir, files)
 
-def num_manifests(mtype=None):
-  destdir, files = list_manifests()
+def num_manifests(mtype=None, dest='local'):
+  destdir, files = list_manifests(dest)
   if mtype:
     files = filter(lambda x: manifest_type(x) == mtype, files)
   return len(files)
 
-def last_manifest():
+def last_manifest(dest='local'):
   '''Returns last backup manifest (directory, filename) pair'''
-  destdir, files = list_manifests()
+  destdir, files = list_manifests(dest)
   latest = files[-1]
   return (destdir, latest)
 
@@ -281,15 +298,15 @@ def manifest_type(fn):
   # fn looks like duplicity-TYPE.DATES.manifest
   return fn.split('.')[0].split('-')[1]
 
-def last_type():
+def last_type(dest='local'):
   '''Returns last backup type, inc or full'''
-  filename = last_manifest()[1]
+  filename = last_manifest(dest)[1]
   return manifest_type(filename)
 
-def last_date_change(to_date):
+def last_date_change(to_date, dest='local'):
   '''Changes the most recent set of duplicity files to look like they were
      from to_date's timestamp'''
-  destdir, latest = last_manifest()
+  destdir, latest = last_manifest(dest)
   olddate = get_manifest_date(latest)
   newdate = subprocess.Popen(['date', '-d', to_date, '+%Y%m%dT%H%M%SZ'], stdout=subprocess.PIPE).communicate()[0].strip()
   cachedir = environ['XDG_CACHE_HOME'] + '/deja-dup/'
@@ -301,16 +318,25 @@ def last_date_change(to_date):
         newname = re.sub(olddate, newdate, f)
         os.rename(f, newname)
 
-def guivisible(frm, obj):
-  if not ldtp.guiexist(frm, obj):
+def guiexist(frm, obj, prefix=False):
+  if not prefix:
+    return obj if ldtp.guiexist(frm, obj) else None
+  else:
+    objs = ldtp.getobjectlist(frm)
+    objs = filter(lambda x: x.startswith(obj), objs)
+    return objs[0] if objs else None
+
+def guivisible(frm, obj, prefix=False):
+  obj = guiexist(frm, obj, prefix)
+  if not obj:
     return False
   states = ldtp.getallstates(frm, obj)
   return ldtp.state.VISIBLE in states
 
-def wait_for_encryption(dlg, obj, max_count):
+def wait_for_encryption(dlg, obj, max_count, prefix=False):
   count = 0
   while count < max_count:
-    if ldtp.guiexist(dlg, obj):
+    if guiexist(dlg, obj, prefix):
       break
     if ldtp.guiexist('dlgAllowaccess'):
       ldtp.click('dlgAllowaccess', 'btnDeny')
@@ -318,30 +344,35 @@ def wait_for_encryption(dlg, obj, max_count):
       ldtp.settextvalue(dlg, 'txtEncryptionpassword', 'test')
       ldtp.click(dlg, 'btnContinue')
     ldtp.wait(1)
+    remap(dlg)
     count += 1
-  assert guivisible(dlg, obj)
+  assert guivisible(dlg, obj, prefix)
 
 def remap(frm):
   ldtp.wait(1) # sometimes (only for newer versions?) ldtp needs a second to catch its breath
   ldtp.remap(frm) # in case this is second time we've run it
 
-def backup_simple(finish=True):
-  ldtp.click('frmDéjàDup', 'btnBackup')
-  assert ldtp.waittillguiexist('dlgBackup')
-  remap('dlgBackup')
-  if guivisible('dlgBackup', 'lblPreferences'):
-    ldtp.click('dlgBackup', 'btnForward')
+def backup_simple(finish=True, error=None, timeout=400):
+  ldtp.click('frmDéjàDup', 'btnBackUp…')
+  assert ldtp.waittillguiexist('dlgBackUp')
+  remap('dlgBackUp')
+  if guivisible('dlgBackUp', 'lblPreferences'):
+    ldtp.click('dlgBackUp', 'btnForward')
     ldtp.wait(1) # give ldtp a second
-    ldtp.click('dlgBackup', 'btnForward')
-  ldtp.click('dlgBackup', 'btnBackup')
-  remap('dlgBackup')
+    ldtp.click('dlgBackUp', 'btnForward')
+  ldtp.click('dlgBackUp', 'btnBackUp')
+  remap('dlgBackUp')
   if finish:
-    wait_for_encryption('dlgBackup', 'lblYourfilesweresuccessfullybackedup', 400)
-    ldtp.click('dlgBackup', 'btnClose')
-    ldtp.waittillguinotexist('dlgBackup')
+    if not error:
+      error = 'lblYourfilesweresuccessfullybackedup'
+      wait_for_encryption('dlgBackUp', error, timeout)
+    else:
+      wait_for_encryption('dlgBackUp', error, timeout, prefix=True)
+    ldtp.click('dlgBackUp', 'btnClose')
+    ldtp.waittillguinotexist('dlgBackUp')
 
 def restore_simple(path, date=None):
-  ldtp.click('frmDéjàDup', 'btnRestore')
+  ldtp.click('frmDéjàDup', 'btnRestore…')
   assert ldtp.waittillguiexist('dlgRestore')
   remap('dlgRestore')
   if ldtp.guiexist('dlgRestore', 'lblPreferences'):
@@ -378,8 +409,12 @@ def restore_specific(files, path, date=None):
     ldtp.comboselect('dlgRestore', 'cboDate', date)
   ldtp.click('dlgRestore', 'btnForward')
   ldtp.click('dlgRestore', 'btnRestore')
-  assert ldtp.waittillguiexist('dlgRestore', 'lblYourfilesweresuccessfullyrestored')
-  assert guivisible('dlgRestore', 'lblYourfilesweresuccessfullyrestored')
+  if len(files) == 1:
+    lbl = 'lblYourfilewassuccessfullyrestored'
+  else:
+    lbl = 'lblYourfilesweresuccessfullyrestored'
+  assert ldtp.waittillguiexist('dlgRestore', lbl)
+  assert guivisible('dlgRestore', lbl)
   ldtp.click('dlgRestore', 'btnClose')
 
 def file_equals(path, contents):
